@@ -2,70 +2,45 @@
 #
 # Author: Yipeng Sun <syp at umd dot edu>
 # License: BSD 2-clause
-# Last Change: Sat Jun 26, 2021 at 12:00 AM +0200
+# Las Change: Tue Aug 31, 2021 at 11:38 PM +0200
 
 import re
 import logging
 
-from collections import defaultdict, Counter
-from dataclasses import dataclass
+from collections import defaultdict
 from copy import deepcopy
 
 from pyBabyMaker.base import TermColor as TC
 from pyBabyMaker.base import UniqueList, BaseMaker
 from pyBabyMaker.base import update_config
 from pyBabyMaker.engine.core import template_transformer, template_evaluator
-from pyBabyMaker.var_resolver import Variable, VariableResolver
+from pyBabyMaker.dag_resolver import resolve_scope
+from pyBabyMaker.dag_resolver import Variable
 
 
 ###########
 # Helpers #
 ###########
 
-@dataclass(repr=False)
-class BabyVariable(Variable):
-    """
-    Store both raw variables and resolved variables.
+class BabyResolver:
+    def __init__(self, scopes, skip_names=[]):
+        self.scopes = scopes
+        self.skip_names = skip_names
+        self.resolved = UniqueList()
 
-    The added attributes make sorting variables easier.
-    """
-    input: bool = False
-    output: bool = True
-    fake: bool = False
-
-    def __post_init__(self):
-        super().__post_init__()
-        self._fname = self.name  # 'fname' -> full name
-        self.fname_set = False
-
-    @property
-    def fname(self):
-        return self._fname
-
-    @fname.setter
-    def fname(self, fname):
-        if not self.fname_set:
-            self.fname_set = True
-            self._fname = fname
-
-
-class BabyVariableResolver(VariableResolver):
-    @staticmethod
-    def format_resolved(scope, var):
-        """
-        Prepare resolved variables for ``babymaker``.
-        """
-        var.fname = scope+'_'+var.name
-        return var
+    def resolve(self, scope,
+                ordering=['literals', 'calculation', 'rename', 'raw'],
+                **kwargs):
+        resolved, unresolved = resolve_scope(
+            scope, self.scopes, ordering, postprocess=self.postprocess,
+            resolved_vars=self.resolved, **kwargs)
+        self.resolved += resolved
+        return resolved, unresolved
 
     @staticmethod
-    def unpack_resolved(var):
-        """
-        Unpack resolved variable to (scope, original variable) tuple for
-        ``babymaker``.
-        """
-        scope = var.fname.split('_')[0]
-        return scope, var.name
+    def postprocess(var, node):
+        node.input = var.input
+        node.output = var.output
 
 
 ########################
@@ -103,11 +78,15 @@ class BabyConfigParser:
             'input_trees': UniqueList(),
         }
         self.parse_headers(self.parsed_config, directive)
-        parsed_literals = {k: BabyVariable(k, literal=v)
+        parsed_literals = {k: Variable(k, literal=v)
                            for k, v in self.literals.items()}
 
         for output_tree, config in self.parsed_config['output'].items():
             input_tree = config['input']
+            try:
+                known_warnings = config['mute']
+            except KeyError:
+                known_warnings = []
 
             try:
                 dumped_tree = self.dumped_ntuple[input_tree]
@@ -126,7 +105,7 @@ class BabyConfigParser:
             config = update_config(self.parsed_config, config, merge=merge)
             namespace = defaultdict(dict)
             namespace['literals'] = parsed_literals
-            namespace['raw'] = {n: BabyVariable(n, t, input=True, output=False)
+            namespace['raw'] = {n: Variable(n, t, input=True, output=False)
                                 for n, t in dumped_tree.items()}
 
             # Load all variables in separate namespaces
@@ -134,29 +113,33 @@ class BabyConfigParser:
             self.parse_calculation(config, namespace)
             self.parse_selection(config, namespace)
 
-            # Initialize a variable resolver
-            if 'skip_names' in config:
-                resolver = BabyVariableResolver(namespace, config['skip_names'])
-            else:
-                resolver = BabyVariableResolver(namespace)
-            if self.debug:
-                self._resolvers.append(resolver)
+            skip_names = config['skip_names'] if 'skip_names' in config else []
+            resolver = BabyResolver(namespace, skip_names)
 
             # Resolve variables needed for selection
-            selection, unresolved_selection = resolver.resolve_scope(
-                'selection', ['literals', 'calculation', 'rename', 'raw'])
+            selection, unresolved_selection = resolver.resolve('selection')
 
             # Resolve all other variables
-            keep, unresolved_keep = resolver.resolve_scope('keep', ['raw'])
-            rename, unresolved_rename = resolver.resolve_scope(
-                'rename', ['raw'])
-            calculation, unresolved_calculation = resolver.resolve_scope(
-                'calculation', ['literals', 'calculation', 'rename', 'raw'])
+            keep, unresolved_keep = resolver.resolve('keep', ['raw'])
+            rename, unresolved_rename = resolver.resolve('rename', ['raw'])
+            calculation, unresolved_calculation = resolver.resolve(
+                'calculation')
             resolved_vars = selection + keep + rename + calculation
             most_unresolved_vars = unresolved_keep + unresolved_rename + \
                 unresolved_calculation
 
             # Warn about variables that can't be resolved
+            most_unresolved_vars = [
+                v for v in most_unresolved_vars
+                if not self.match(known_warnings, v.name)]
+            unresolved_selection = [
+                v for v in unresolved_selection
+                if not self.match(known_warnings, v.rval)]
+
+            for var in unresolved_selection:
+                print("{}Selection expr {} cannot be resolved...{}".format(
+                    TC.YELLOW, var.rval, TC.END))
+
             for var in most_unresolved_vars:
                 if var.output:
                     print("{}Output branch {} cannot be resolved...{}".format(
@@ -165,24 +148,6 @@ class BabyConfigParser:
                     print("{}Temp variable {} cannot be resolved...{}".format(
                         TC.YELLOW, var.name, TC.END))
 
-            for var in unresolved_selection:
-                print("{}Selection expr {} cannot be resolved...{}".format(
-                    TC.YELLOW, var.rval, TC.END))
-
-            # Error when one output branch has multiple definitions
-            output_br = [v for v in resolved_vars if v.output]
-            dupl_br_name = [k for k, v in Counter(
-                [n.name for n in output_br]).items() if v > 1]
-            dupl_br_def = {n: [x for x in output_br if x.name == n]
-                           for n in dupl_br_name}
-            if dupl_br_def:
-                raise ValueError(
-                    'These output branches are defined multiple times:\n' +
-                    '\n'.join(
-                        ['\n'.join(['  '+n+':']+['    '+str(v) for v in vals])
-                         for n, vals in dupl_br_def.items()])
-                )
-
             directive['trees'][output_tree] = {
                 'input_tree': input_tree,
                 'sel': ['true']+[v.rval for v in selection if v.fake],
@@ -233,12 +198,11 @@ class BabyConfigParser:
 
             if 'rename' in config and var.name in config['rename']:
                 renamed_var = rename_dict[var.name]
-                namespace['rename'][renamed_var] = BabyVariable(
+                namespace['rename'][renamed_var] = Variable(
                     renamed_var, var.type, [var.name])
-                continue
 
             if 'keep' in config and cls.match(config['keep'], var.name):
-                namespace['keep'][var.name] = BabyVariable(
+                namespace['keep'][var.name] = Variable(
                     var.name, var.type, [var.name])
 
     @staticmethod
@@ -248,8 +212,8 @@ class BabyConfigParser:
         """
         if 'calculation' in config:
             for name, code in config['calculation'].items():
-                datatype, *rvalues = [i.strip() for i in code.split(';')]
-                if not rvalues:
+                datatype, *rvals = [i.strip() for i in code.split(';')]
+                if not rvals:
                     raise ValueError('Illegal specification for {}: {}.'.format(
                         name, code
                     ))
@@ -259,8 +223,8 @@ class BabyConfigParser:
                     datatype = datatype.strip('^')
                     output = False
 
-                namespace['calculation'][name] = BabyVariable(
-                    name, datatype, rvalues, output=output)
+                namespace['calculation'][name] = Variable(
+                    name, datatype, rvals, output=output)
 
     @classmethod
     def parse_selection(cls, config, namespace):
@@ -274,9 +238,8 @@ class BabyConfigParser:
             selections += config['selection']
 
         for idx, expr in enumerate(selections):
-            namespace['selection']['sel'+str(idx)] = BabyVariable(
-                'sel'+str(idx), rvalues=[expr],
-                input=False, output=False, fake=True)
+            namespace['selection']['sel'+str(idx)] = Variable(
+                'sel'+str(idx), rvals=[expr], input=False, output=False)
 
     @staticmethod
     def match(patterns, string, return_value=True):
@@ -336,6 +299,19 @@ class BabyMaker(BaseMaker):
         if self.use_reformatter:
             self.reformat(filename)
 
+    def debug(self, filename, literals={}, blocked_trees=[], debug=False):
+        """
+        Generate a debug file for the directives that will be used for C++
+        generation.
+        """
+        parsed_config = self.read(self.config_filename)
+        dumped_ntuple, _ = self.dump_ntuples(blocked_trees)
+        directive = self.directive_gen(
+            parsed_config, dumped_ntuple, literals, debug)
+
+        with open(filename, 'w') as f:
+            f.write(self.directive_debug(directive))
+
     def dump_ntuples(self, blocked_trees=[]):
         """
         Dump main ntuple and all friend ntuples.
@@ -367,3 +343,41 @@ class BabyMaker(BaseMaker):
         """
         parser = BabyConfigParser(parsed_config, dumped_ntuple, literals, debug)
         return parser.parse()
+
+    @staticmethod
+    def directive_debug(directive):
+        """
+        Generate a plain-text representation of the directive.
+
+        Currently we only generate the 'trees' part
+        """
+        output = ''
+
+        for tree, val in directive['trees'].items():
+            output += '# {}, from {}\n\n'.format(tree, val['input_tree'])
+
+            output += '## Selection-related\n\n'
+            for key, repl in [('sel', 'Cuts'),
+                              ('pre_sel_vars', 'Pre-cut variables'),
+                              ('post_sel_vars', 'Post-cut variables')]:
+                output += '### {}\n'.format(repl)
+                for i in val[key]:
+                    output += ' - {}\n'.format(i)
+                output += '\n'
+
+            output += '## Input, output and temp variables\n\n'
+            for key, repl in [('input', 'Input variables'),
+                              ('output', 'Output variables'),
+                              ('tmp', 'Temp variables')]:
+                output += '### {}\n'.format(repl)
+                for i in val[key]:
+                    output += ' - {}\n'.format(i)
+                output += '\n'
+
+            output += '## Input variable full names\n'
+            for i in val['input_br']:
+                output += ' - {}\n'.format(i)
+
+            output += '\n'
+
+        return output
