@@ -2,11 +2,12 @@
 #
 # Author: Yipeng Sun <syp at umd dot edu>
 # License: BSD 2-clause
-# Last Change: Thu Sep 03, 2020 at 11:51 PM +0800
+# Last Change: Mon Jan 11, 2021 at 02:55 AM +0100
 """
 This module provides simple parsed boolean tree info extraction
 """
 
+from pyBabyMaker.base import UniqueList
 from .syntax import cpp_boolean_parser as cpp
 
 
@@ -16,9 +17,12 @@ def find_all_args(expr):
 
     :param str expr: Expression to be parsed
     """
+    if not len(expr):
+        return []
+
     tree = cpp.parse(expr)
     args = tree.find_data('arguments')
-    result = []
+    result = UniqueList()
 
     for subtree in args:
         result += [t.children[0].value for t in subtree.find_data('var')]
@@ -32,5 +36,9 @@ def find_all_vars(expr):
 
     :param str expr: Expression to be parsed
     """
+    if not len(expr):
+        return []
+
     tree = cpp.parse(expr)
-    return [t.children[0].value for t in tree.find_data('var')]
+    return UniqueList([t.children[0].value for t in tree.find_data('var')])
+    # This is needed otherwise 'b*b' -> ['b', 'b']
