@@ -2,7 +2,7 @@
 #
 # Author: Yipeng Sun <syp at umd dot edu>
 # License: BSD 2-clause
-# Last Change: Mon Jun 21, 2021 at 04:24 PM +0200
+# Last Change: Wed Sep 01, 2021 at 11:22 PM +0200
 
 import yaml
 import pytest
@@ -12,8 +12,8 @@ from os import pardir
 from os.path import join as J
 from os.path import dirname, realpath
 
-from pyBabyMaker.babymaker import BabyVariable, BabyMaker, BabyConfigParser, \
-    BabyVariableResolver
+from pyBabyMaker.babymaker import BabyMaker, BabyConfigParser, BabyResolver
+from pyBabyMaker.dag_resolver import Node, Variable
 from pyBabyMaker.base import UniqueList
 from pyBabyMaker.io.NestedYAMLLoader import NestedYAMLLoader
 from pyBabyMaker.io.TupleDump import PyTupleDump
@@ -27,30 +27,6 @@ SAMPLE_TMPL   = J(PARDIR, 'pyBabyMaker', 'cpp_templates', 'babymaker.cpp')
 SAMPLE_CPP    = J(PARDIR, 'samples', 'sample-babymaker.cpp')
 
 
-######################
-# Variable container #
-######################
-
-def test_BabyVariable_default():
-    var = BabyVariable('stuff')
-
-    assert var.input is False
-    assert var.output is True
-    assert var.rval == ''
-    assert str(var) == 'nil stuff = '
-
-
-def test_BabyVariable_set_fname():
-    var = BabyVariable('stuff')
-    assert var.fname == 'stuff'
-
-    var.fname = 'test_stuff'
-    assert var.fname == 'test_stuff'
-
-    var.fname = 'fake_stuff'
-    assert var.fname == 'test_stuff'
-
-
 #############################
 # Test BabyMaker as a whole #
 #############################
@@ -64,7 +40,6 @@ def test_BabyMaker_cpp_gen(tmp_path):
                        for line in gen_cpp.read_text().split('\n')[1:]]
 
     with open(SAMPLE_CPP, 'r') as f:
-        print(f)
         assert gen_cpp_content == [line.strip() for line in f.readlines()]
 
 
@@ -93,61 +68,67 @@ def test_BabyConfigParser_parse_ATuple(realistic_BabyConfigParser):
         'raw_Y_ISOLATION_BDT',
         'raw_Y_PT',
         'raw_Y_PE',
-        'raw_runNumber',
-        'raw_eventNumber',
-        'raw_GpsTime',
         'raw_Y_PX',
         'raw_Y_PY',
         'raw_Y_PZ',
+        'raw_runNumber',
+        'raw_eventNumber',
+        'raw_GpsTime',
         'raw_D0_P',
     ]
     assert directive['trees']['ATuple']['input'] == [
-        BabyVariable('Y_ISOLATION_BDT', 'double', input=True, output=False),
-        BabyVariable('Y_PT', 'double', input=True, output=False),
-        BabyVariable('Y_PE', 'double', input=True, output=False),
-        BabyVariable('runNumber', 'UInt_t', input=True, output=False),
-        BabyVariable('eventNumber', 'ULong64_t', input=True, output=False),
-        BabyVariable('GpsTime', 'ULong64_t', input=True, output=False),
-        BabyVariable('Y_PX', 'double', input=True, output=False),
-        BabyVariable('Y_PY', 'double', input=True, output=False),
-        BabyVariable('Y_PZ', 'double', input=True, output=False),
-        BabyVariable('D0_P', 'double', input=True, output=False),
+        Node('Y_ISOLATION_BDT', 'raw', 'double', input=True, output=False),
+        Node('Y_PT', 'raw', 'double', input=True, output=False),
+        Node('Y_PE', 'raw', 'double', input=True, output=False),
+        Node('Y_PX', 'raw', 'double', input=True, output=False),
+        Node('Y_PY', 'raw', 'double', input=True, output=False),
+        Node('Y_PZ', 'raw', 'double', input=True, output=False),
+        Node('runNumber', 'raw', 'UInt_t', input=True, output=False),
+        Node('eventNumber', 'raw', 'ULong64_t', input=True, output=False),
+        Node('GpsTime', 'raw', 'ULong64_t', input=True, output=False),
+        Node('D0_P', 'raw', 'double', input=True, output=False),
     ]
     assert directive['trees']['ATuple']['output'] == [
-        BabyVariable('Y_PE', 'double', ['Y_PE']),
-        BabyVariable('runNumber', 'UInt_t', ['runNumber']),
-        BabyVariable('eventNumber', 'ULong64_t', ['eventNumber']),
-        BabyVariable('GpsTime', 'ULong64_t', ['GpsTime']),
-        BabyVariable('y_pt', 'double', ['Y_PT']),
-        BabyVariable('y_px', 'double', ['Y_PX']),
-        BabyVariable('y_py', 'double', ['Y_PY']),
-        BabyVariable('y_pz', 'double', ['Y_PZ']),
-        BabyVariable('RandStuff', 'double', ['TempStuff*pi']),
-        BabyVariable('some_other_var', 'double', ['some_var*pi']),
-        BabyVariable('alt_def', 'double', ['b0_pe', 'Y_PE']),
+        Node('Y_PT', 'keep', 'double', 'Y_PT'),
+        Node('Y_PE', 'keep', 'double', 'Y_PE'),
+        Node('Y_PX', 'keep', 'double', 'Y_PX'),
+        Node('Y_PY', 'keep', 'double', 'Y_PY'),
+        Node('Y_PZ', 'keep', 'double', 'Y_PZ'),
+        Node('runNumber', 'keep', 'UInt_t', 'runNumber'),
+        Node('eventNumber', 'keep', 'ULong64_t', 'eventNumber'),
+        Node('GpsTime', 'keep', 'ULong64_t', 'GpsTime'),
+        Node('y_pt', 'rename', 'double', 'Y_PT'),
+        Node('y_px', 'rename', 'double', 'Y_PX'),
+        Node('y_py', 'rename', 'double', 'Y_PY'),
+        Node('y_pz', 'rename', 'double', 'Y_PZ'),
+        Node('RandStuff', 'calculation', 'double', 'TempStuff*pi'),
+        Node('some_other_var', 'calculation', 'double', 'some_var*pi'),
+        Node('alt_def', 'calculation', 'double', 'Y_PE'),
     ]
     assert directive['trees']['ATuple']['tmp'] == [
-        BabyVariable('TempStuff', 'double', ['D0_P+Y_PT'], output=False),
-        BabyVariable('some_var', 'double', ['y_pt + y_pz'], output=False),
+        Node('TempStuff', 'calculation', 'double', 'D0_P+Y_PT', output=False),
+        Node('some_var', 'calculation', 'double', 'y_pt + y_pz', output=False),
     ]
     assert directive['trees']['ATuple']['pre_sel_vars'] == []
     assert directive['trees']['ATuple']['post_sel_vars'] == [
-        BabyVariable('Y_PE', 'double', ['Y_PE']),
-        BabyVariable('runNumber', 'UInt_t', ['runNumber']),
-        BabyVariable('eventNumber', 'ULong64_t', ['eventNumber']),
-        BabyVariable('GpsTime', 'ULong64_t', ['GpsTime']),
-        BabyVariable('y_pt', 'double', ['Y_PT']),
-        BabyVariable('y_px', 'double', ['Y_PX']),
-        BabyVariable('y_py', 'double', ['Y_PY']),
-        BabyVariable('y_pz', 'double', ['Y_PZ']),
-        BabyVariable('TempStuff', 'double', ['D0_P+Y_PT'], output=False),
-        BabyVariable('RandStuff', 'double', ['TempStuff*pi']),
-        BabyVariable('some_var', 'double', ['y_pt + y_pz'], output=False),
-        BabyVariable('some_other_var', 'double', ['some_var*pi']),
-        BabyVariable('alt_def', 'double', ['b0_pe', 'Y_PE']),
+        Node('Y_PT', 'keep', 'double', 'Y_PT'),
+        Node('Y_PE', 'keep', 'double', 'Y_PE'),
+        Node('Y_PX', 'keep', 'double', 'Y_PX'),
+        Node('Y_PY', 'keep', 'double', 'Y_PY'),
+        Node('Y_PZ', 'keep', 'double', 'Y_PZ'),
+        Node('runNumber', 'keep', 'UInt_t', 'runNumber'),
+        Node('eventNumber', 'keep', 'ULong64_t', 'eventNumber'),
+        Node('GpsTime', 'keep', 'ULong64_t', 'GpsTime'),
+        Node('y_pt', 'rename', 'double', 'Y_PT'),
+        Node('y_px', 'rename', 'double', 'Y_PX'),
+        Node('y_py', 'rename', 'double', 'Y_PY'),
+        Node('y_pz', 'rename', 'double', 'Y_PZ'),
+        Node('TempStuff', 'calculation', 'double', 'D0_P+Y_PT', output=False),
+        Node('RandStuff', 'calculation', 'double', 'TempStuff*pi'),
+        Node('some_var', 'calculation', 'double', 'y_pt + y_pz', output=False),
+        Node('some_other_var', 'calculation', 'double', 'some_var*pi'),
+        Node('alt_def', 'calculation', 'double', 'Y_PE'),
     ]
-    assert realistic_BabyConfigParser._resolvers[0]._resolved_names[0] == \
-        ('raw', 'Y_ISOLATION_BDT')
 
 
 def test_BabyConfigParser_parse_AnotherTuple(realistic_BabyConfigParser):
@@ -156,33 +137,34 @@ def test_BabyConfigParser_parse_AnotherTuple(realistic_BabyConfigParser):
     assert directive['trees']['AnotherTuple']['input_tree'] == \
         'TupleB0/DecayTree'
     assert directive['trees']['AnotherTuple']['input'] == [
-        BabyVariable('Y_ISOLATION_BDT', 'double', input=True, output=False),
-        BabyVariable('Y_PT', 'double', input=True, output=False),
-        BabyVariable('Y_PE', 'double', input=True, output=False),
-        BabyVariable('Y_PX', 'double', input=True, output=False),
-        BabyVariable('Y_PY', 'double', input=True, output=False),
-        BabyVariable('Y_PZ', 'double', input=True, output=False),
-        BabyVariable('runNumber', 'UInt_t', input=True, output=False),
-        BabyVariable('eventNumber', 'ULong64_t', input=True, output=False),
-        BabyVariable('GpsTime', 'ULong64_t', input=True, output=False),
-        BabyVariable('D0_P', 'double', input=True, output=False),
+        Node('Y_ISOLATION_BDT', 'raw', 'double', input=True, output=False),
+        Node('Y_PT', 'raw', 'double', input=True, output=False),
+        Node('Y_PE', 'raw', 'double', input=True, output=False),
+        Node('Y_PX', 'raw', 'double', input=True, output=False),
+        Node('Y_PY', 'raw', 'double', input=True, output=False),
+        Node('Y_PZ', 'raw', 'double', input=True, output=False),
+        Node('runNumber', 'raw', 'UInt_t', input=True, output=False),
+        Node('eventNumber', 'raw', 'ULong64_t', input=True, output=False),
+        Node('GpsTime', 'raw', 'ULong64_t', input=True, output=False),
+        Node('D0_P', 'raw', 'double', input=True, output=False),
     ]
     assert directive['trees']['AnotherTuple']['output'] == [
         # NOTE: 'some_other_var' is not resolvable for this tree!
         #       Because the change in 'rename' selection, 'y_pt' and 'y_pz' are
         #       not defined!
-        BabyVariable('b0_pt', 'double', ['Y_PT']),
-        BabyVariable('Y_PE', 'double', ['Y_PE']),
-        BabyVariable('Y_PX', 'double', ['Y_PX']),
-        BabyVariable('Y_PY', 'double', ['Y_PY']),
-        BabyVariable('Y_PZ', 'double', ['Y_PZ']),
-        BabyVariable('runNumber', 'UInt_t', ['runNumber']),
-        BabyVariable('eventNumber', 'ULong64_t', ['eventNumber']),
-        BabyVariable('GpsTime', 'ULong64_t', ['GpsTime']),
-        BabyVariable('RandStuff', 'double', ['TempStuff*pi']),
+        Node('b0_pt', 'rename', 'double', 'Y_PT'),
+        Node('Y_PT', 'keep', 'double', 'Y_PT'),
+        Node('Y_PE', 'keep', 'double', 'Y_PE'),
+        Node('Y_PX', 'keep', 'double', 'Y_PX'),
+        Node('Y_PY', 'keep', 'double', 'Y_PY'),
+        Node('Y_PZ', 'keep', 'double', 'Y_PZ'),
+        Node('runNumber', 'keep', 'UInt_t', 'runNumber'),
+        Node('eventNumber', 'keep', 'ULong64_t', 'eventNumber'),
+        Node('GpsTime', 'keep', 'ULong64_t', 'GpsTime'),
+        Node('RandStuff', 'calculation', 'double', 'TempStuff*pi'),
     ]
     assert directive['trees']['AnotherTuple']['tmp'] == [
-        BabyVariable('TempStuff', 'double', ['D0_P+Y_PT'], output=False),
+        Node('TempStuff', 'calculation', 'double', 'D0_P+Y_PT', output=False),
     ]
     assert directive['system_headers'] == ['cmath', 'iostream']
     assert directive['trees']['AnotherTuple']['sel'] == [
@@ -226,7 +208,7 @@ def directive():
 def make_namespace():
     def namespace(dumped_tree):
         ns = defaultdict(dict)
-        ns['raw'] = {n: BabyVariable(n, t, input=True)
+        ns['raw'] = {n: Variable(n, t, input=True)
                      for n, t in dumped_tree.items()}
 
         return ns
@@ -242,29 +224,6 @@ def test_BabyConfigParser_parse_non_existing_tree(directive):
     assert directive['trees'] == {}
 
 
-def test_BabyConfigParser_parse_duplicated_output_br():
-    parsed_yml = {'output': {
-        'test_tree': {
-            'input': 'tree',
-            'rename': {'Y_PX': 'dupl'},
-            'calculation': {'dupl': 'float;Y_PZ'}
-        }}}
-    dumped_ntuple = {'tree': {
-        'Y_PX': 'float',
-        'Y_PY': 'float',
-        'Y_PZ': 'float'
-    }}
-    parser = BabyConfigParser(parsed_yml, dumped_ntuple)
-
-    with pytest.raises(ValueError) as e:
-        parser.parse()
-    assert e.value.args[0] == \
-        'These output branches are defined multiple times:\n' \
-        '  dupl:\n' \
-        '    float dupl = Y_PX\n' \
-        '    float dupl = Y_PZ'
-
-
 def test_BabyConfigParser_parse_headers_none(directive):
     BabyConfigParser.parse_headers({}, directive)
 
@@ -315,10 +274,10 @@ def test_BabyConfigParser_parse_drop_keep_rename(make_namespace):
     BabyConfigParser.parse_drop_keep_rename(config, namespace)
 
     assert namespace['keep'] == {
-        n: BabyVariable(n, 'float', [n]) for n in dumped_tree.keys()
+        n: Variable(n, 'float', [n]) for n in dumped_tree.keys()
         if not n.startswith('Y') and n != 'Z_PZ' and n != 'Z_PY'}
     assert namespace['rename'] == {
-        'z_py': BabyVariable('z_py', 'float', ['Z_PY'])}
+        'z_py': Variable('z_py', 'float', ['Z_PY'])}
 
 
 def test_BabyConfigParser_parse_calculation(make_namespace):
@@ -330,9 +289,9 @@ def test_BabyConfigParser_parse_calculation(make_namespace):
     BabyConfigParser.parse_calculation(config, namespace)
 
     assert namespace['calculation'] == {
-        'Y_P_TEMP': BabyVariable(
+        'Y_P_TEMP': Variable(
             'Y_P_TEMP', 'double', ['Y_PX+1'], output=False),
-        'Y_P_shift': BabyVariable('Y_P_shift', 'double', ['Y_P_TEMP'])
+        'Y_P_shift': Variable('Y_P_shift', 'double', ['Y_P_TEMP'])
     }
 
 
@@ -344,7 +303,7 @@ def test_BabyConfigParser_parse_calculation_alt(make_namespace):
     BabyConfigParser.parse_calculation(config, namespace)
 
     assert namespace['calculation'] == {
-        'Y_P_TEMP': BabyVariable(
+        'Y_P_TEMP': Variable(
             'Y_P_TEMP', 'double', ['Y_PX+1', 'FUNC(Y_PX, 1)'], output=False)}
 
 
@@ -364,59 +323,6 @@ def test_BabyConfigParser_parse_calculation_invalid_spec(make_namespace):
 # Test variable resolution #
 ############################
 
-def test_BabyVariableResolver_simple(make_namespace):
-    dumped_tree = {
-        'Y_PX': 'float',
-        'Y_PY': 'float',
-        'Y_PZ': 'float',
-        'Y_PT': 'float',
-        'Y_PE': 'float',
-    }
-    namespace = make_namespace(dumped_tree)
-    var = BabyVariable('p_sum', 'float', ['Y_PX+Y_PY'])
-    resolver = BabyVariableResolver(namespace)
-    status, load_seq, _ = resolver.resolve_var('test', var)
-
-    assert status is True
-    assert load_seq == [
-        namespace['raw']['Y_PX'],
-        namespace['raw']['Y_PY'],
-        var
-    ]
-    assert load_seq[2].rval == 'raw_Y_PX+raw_Y_PY'
-    assert load_seq[2].name == 'p_sum'
-    assert load_seq[2].fname == 'test_p_sum'
-
-
-def test_BabyVariableResolver_self(make_namespace):
-    dumped_tree = {
-        'q2': 'float',
-    }
-    namespace = make_namespace(dumped_tree)
-    var = BabyVariable('q2', 'float', ['GEV2(q2)'])
-    namespace['test']['q2'] = var
-    resolver = BabyVariableResolver(namespace)
-    status, load_seq, _ = resolver.resolve_var(
-        'test', var, ['test', 'raw'])
-
-    assert status is True
-    assert load_seq == [
-        namespace['raw']['q2'],
-        var
-    ]
-
-
-def test_BabyVariableResolver_self_no_resolve(make_namespace):
-    namespace = make_namespace({})
-    var = BabyVariable('q2', 'float', ['GEV2(q2)'])
-    namespace['test']['q2'] = var
-    resolver = BabyVariableResolver(namespace)
-    status, _, _ = resolver.resolve_var(
-        'test', var, ['test', 'raw'])
-
-    assert status is False
-
-
 def test_BabyVariableResolver_scope_keep(make_namespace):
     dumped_tree = {
         'q2': 'float',
@@ -426,17 +332,17 @@ def test_BabyVariableResolver_scope_keep(make_namespace):
     }
     namespace = make_namespace(dumped_tree)
     namespace['keep'] = {
-        'Y_PX': BabyVariable('Y_PX', 'float', ['Y_PX']),
-        'q2': BabyVariable('q2', 'float', ['q2'])
+        'Y_PX': Variable('Y_PX', 'float', ['Y_PX']),
+        'q2': Variable('q2', 'float', ['q2'])
     }
-    resolver = BabyVariableResolver(namespace)
-    load_seq, unresolved = resolver.resolve_scope('keep', ordering=['raw'])
+    resolver = BabyResolver(namespace)
+    load_seq, unresolved = resolver.resolve('keep', ordering=['raw'])
 
     assert load_seq == [
-        namespace['raw']['Y_PX'],
-        namespace['keep']['Y_PX'],
-        namespace['raw']['q2'],
-        namespace['keep']['q2'],
+        Node('Y_PX', 'raw', 'float'),
+        Node('Y_PX', 'keep', 'float', 'Y_PX'),
+        Node('q2', 'raw', 'float'),
+        Node('q2', 'keep', 'float', 'q2'),
     ]
     assert unresolved == []
 
@@ -450,17 +356,17 @@ def test_BabyVariableResolver_scope_rename(make_namespace):
     }
     namespace = make_namespace(dumped_tree)
     namespace['rename'] = {
-        'y_px': BabyVariable('y_px', 'float', ['Y_PX']),
-        'Q2': BabyVariable('Q2', 'float', ['q2'])
+        'y_px': Variable('y_px', 'float', ['Y_PX']),
+        'Q2': Variable('Q2', 'float', ['q2'])
     }
-    resolver = BabyVariableResolver(namespace)
-    load_seq, unresolved = resolver.resolve_scope('rename', ordering=['raw'])
+    resolver = BabyResolver(namespace)
+    load_seq, unresolved = resolver.resolve('rename', ordering=['raw'])
 
     assert load_seq == [
-        namespace['raw']['Y_PX'],
-        namespace['rename']['y_px'],
-        namespace['raw']['q2'],
-        namespace['rename']['Q2'],
+        Node('Y_PX', 'raw', 'float'),
+        Node('y_px', 'rename', 'float', 'Y_PX'),
+        Node('q2', 'raw', 'float'),
+        Node('Q2', 'rename', 'float', 'q2'),
     ]
     assert load_seq[3].rval == 'raw_q2'
     assert load_seq[3].name == 'Q2'
@@ -477,18 +383,18 @@ def test_BabyVariableResolver_scope_calculation(make_namespace):
     }
     namespace = make_namespace(dumped_tree)
     namespace['calc'] = {
-        'y_p_sum': BabyVariable('y_p_sum', 'float', ['Y_PX+Y_PY']),
-        'q2': BabyVariable('q2', 'float', ['q2 / 1000'])
+        'y_p_sum': Variable('y_p_sum', 'float', ['Y_PX+Y_PY']),
+        'q2': Variable('q2', 'float', ['q2 / 1000'])
     }
-    resolver = BabyVariableResolver(namespace)
-    load_seq, unresolved = resolver.resolve_scope('calc')
+    resolver = BabyResolver(namespace)
+    load_seq, unresolved = resolver.resolve('calc')
 
     assert load_seq == [
-        namespace['raw']['Y_PX'],
-        namespace['raw']['Y_PY'],
-        namespace['calc']['y_p_sum'],
-        namespace['raw']['q2'],
-        namespace['calc']['q2'],
+        Node('Y_PX', 'raw', 'float'),
+        Node('Y_PY', 'raw', 'float'),
+        Node('y_p_sum', 'calc', 'float', 'Y_PX+Y_PY'),
+        Node('q2', 'raw', 'float'),
+        Node('q2', 'calc', 'float', 'q2 / 1000'),
     ]
     assert load_seq[2].rval == 'raw_Y_PX+raw_Y_PY'
     assert load_seq[4].rval == 'raw_q2 / 1000'
@@ -503,18 +409,18 @@ def test_BabyVariableResolver_scope_calculation_complex(make_namespace):
     }
     namespace = make_namespace(dumped_tree)
     namespace['calc'] = {
-        'q2_diff': BabyVariable('q2_diff', 'float', ['q2_temp']),
-        'q2_temp': BabyVariable('q2_temp', 'float', ['q2 / 1000'], output=False)
+        'q2_diff': Variable('q2_diff', 'float', ['q2_temp']),
+        'q2_temp': Variable('q2_temp', 'float', ['q2 / 1000'], output=False)
     }
-    resolver = BabyVariableResolver(namespace)
-    load_seq, unresolved = resolver.resolve_scope('calc')
+    resolver = BabyResolver(namespace)
+    load_seq, unresolved = resolver.resolve('calc', ordering=['calc', 'raw'])
 
     assert load_seq == [
-        namespace['raw']['q2'],
-        namespace['calc']['q2_temp'],
-        namespace['calc']['q2_diff'],
+        Node('q2', 'raw', 'float'),
+        Node('q2_temp', 'calc', 'float', 'q2 / 1000', output=False),
+        Node('q2_diff', 'calc', 'float', 'q2_temp'),
     ]
-    assert str(load_seq[2]) == 'float q2_diff = q2_temp'
+    assert str(load_seq[2]) == 'float calc_q2_diff = calc_q2_temp'
     assert load_seq[2].fname == 'calc_q2_diff'
     assert load_seq[2].rval == 'calc_q2_temp'
     assert unresolved == []
@@ -528,23 +434,24 @@ def test_BabyVariableResolver_scope_calculation_more_complex(make_namespace):
     }
     namespace = make_namespace(dumped_tree)
     namespace['rename'] = {
-        'y_px': BabyVariable('y_px', 'float', ['Y_PX']),
-        'y_py': BabyVariable('y_py', 'float', ['Y_PY']),
+        'y_px': Variable('y_px', 'float', ['Y_PX']),
+        'y_py': Variable('y_py', 'float', ['Y_PY']),
     }
     namespace['calc'] = {
-        'y_p_sum': BabyVariable('y_p_sum', 'float', ['y_p_temp']),
-        'y_p_temp': BabyVariable('y_p_temp', 'float', ['y_px + y_py'])
+        'y_p_sum': Variable('y_p_sum', 'float', ['y_p_temp']),
+        'y_p_temp': Variable('y_p_temp', 'float', ['y_px + y_py'])
     }
-    resolver = BabyVariableResolver(namespace)
-    load_seq, unresolved = resolver.resolve_scope('calc')
+    resolver = BabyResolver(namespace)
+    load_seq, unresolved = resolver.resolve(
+        'calc', ordering=['calc', 'rename', 'raw'])
 
     assert load_seq == [
-        namespace['raw']['Y_PX'],
-        namespace['rename']['y_px'],
-        namespace['raw']['Y_PY'],
-        namespace['rename']['y_py'],
-        namespace['calc']['y_p_temp'],
-        namespace['calc']['y_p_sum'],
+        Node('Y_PX', 'raw', 'float'),
+        Node('y_px', 'rename', 'float', 'Y_PX'),
+        Node('Y_PY', 'raw', 'float'),
+        Node('y_py', 'rename', 'float', 'Y_PY'),
+        Node('y_p_temp', 'calc', 'float', 'y_px + y_py'),
+        Node('y_p_sum', 'calc', 'float', 'y_p_temp')
     ]
     assert unresolved == []
 
